package com.ym.mec.biz.dal.dao;


import com.ym.mec.biz.dal.entity.SysMusicScoreCategories;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;
import java.util.Set;

public interface SysMusicScoreCategoriesDao extends BaseDAO<Integer, SysMusicScoreCategories> {

    List<SysMusicScoreCategories> findByParentId(@Param("parentId") Integer parentId,@Param("search") String search, @Param("enable") Boolean enable);

    void batchInsert(@Param("categoriesList") List<SysMusicScoreCategories> sysMusicScoreCategoriesList, @Param("organId") String organId, @Param("enable") Boolean enable);

    void batchDel(@Param("categoriesIds") String categoriesIds);

    String findByCategoriesIds(@Param("categoriesIds") String categoriesIds);

    void batchUpdate(@Param("categories") List<SysMusicScoreCategories> updateCategories, @Param("organId") String organId, @Param("enable") Boolean enable);

    void enable(@Param("categoriesIds") String categoriesIds, @Param("enable") boolean enable);

    List<Map<Integer, String>> queryOrganByIds(@Param("categoriesIds") Set<Integer> categoriesIds);
    
    List<SysMusicScoreCategories> queryByIds(String ids);
    
    SysMusicScoreCategories queryByParentId(Integer parentId);
}